import java.awt.Dimension;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import javax.swing.JComponent;
import javax.swing.JFrame;


/**
 * @author tlamatini FJ SEVILLA
 *
 */
public class IsingModel extends JComponent implements Runnable {
	private static final int N = 100;// Lattice Length
	private static final double Coupling = -0.5;
	private static final double phiIsing = 0.0;
		public static void main(String[] args) {
			GeneralFrame f = new GeneralFrame();
			f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			f.add(new IsingModel());
			f.setTitle("2D ISING MODEL");
			f.setVisible(true);
	  }
	
  
  private LatticeSite Sites[][];
  private double energyold;
  private double energynew;

  
  public IsingModel() {

	  Sites = new LatticeSite[N][N];
	  iniconds(Sites);
	  double energyold = energyTotal(Sites);
	  double energynew = 0.0;

    Thread t = new Thread(this);
    t.start();
  }

  public void run() {
    try {
      while (true) {
        repaint();
        Thread.sleep(100);
      }
    } catch (InterruptedException ie) {
    }
  }

  public void paintComponent(Graphics g){
		super.paintComponent(g);
		Graphics2D g2 = (Graphics2D) g; 
		Toolkit kit = Toolkit.getDefaultToolkit();
		Dimension screensize = kit.getScreenSize();
		int screenheight = (int) screensize.height * 4 / 5;
		int screenwidth = (int) screensize.width / 2;
		
		for(int i=0;i<N;i++){
		    int kxp = i + 1;
		    int kxm = i - 1;
		    if(i==N-1){
		      kxp = 0;
		    }
		    if(i==0){
		      kxm = N-1;
		    }
		    for(int j=0;j<N;j++){
			int kyp = j + 1;
			int kym = j - 1;
			if(j==N-1){
			  kyp = 0;
			}
			if(j==0){
			  kym = N-1;
			}
			double Sijz = -1.0 * Sites[i][j].getZdirection();
			energynew = energyold + Coupling * Sijz * (Sites[kxp][j].getZdirection() + Sites[kxm][j].getZdirection() + Sites[i][kyp].getZdirection() + Sites[i][kym].getZdirection());
			double r = Math.random();
			double Pr = Math.exp(energyold-energynew);
			//System.out.println(Pr);
			if(Pr >= r){
			    energyold = energynew;
			    Sites[i][j].setZdirection(Sijz);
			}
		    }
		}
		for(int i=0;i<N;i++){
		    for(int j=0;j<N;j++){
			Rectangle2D.Double siteij = new Rectangle2D.Double((double)(screenwidth/(N+1))*i+1,(double)(screenwidth/(N+1))*j+1,(double)(screenheight/(N+1)),(double)(screenheight/(N+1)));
			g2.setPaint(Sites[i][j].getColor());
			g2.fill(siteij);
		    }
		}
	}
  
  public static void iniconds(LatticeSite sites[][]){
	  	for(int i=0;i<N;i++){
		    for(int j=0;j<N;j++){
			sites[i][j] = new LatticeSite(randomThetaIsing(),phiIsing);
			//sites[i][j] = new LatticeSite(0.0,phiIsing);
		    }
		}
  }

  public static double energyTotal(LatticeSite sites[][]){
		double sum = 0.0;
		int kx,ky;
		for(int i=0;i<N;i++){
		    kx = i + 1;
		    if(i==N-1){
		      kx = 0;
		    }
		    for(int j=0;j<N;j++){
		      ky = j + 1;  
		      if(j==N-1){
			 ky = 0; 
		      }
		      sum = sum + sites[i][j].getZdirection() * (sites[kx][j].getZdirection() + sites[i][ky].getZdirection());
		    }
		}
		return sum * Coupling;
  }

  public static double randomThetaIsing(){
	double zz;
	zz = Math.PI * Math.random();
	if(zz<=Math.PI/2){
	  zz = 0.0;
	}
	else
	  zz = Math.PI;
	return zz;
  }

}